// skills.cpp: implementation of the AOCMySkills class.
//
//////////////////////////////////////////////////////////////////////

#include "skills.h"
#include "../common/aoc_names.h"
#include "../common/aoc_proc.h"
#include <qmessagebox.h>
#include <qfile.h>
#include <qdir.h>


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

AOCMySkills::AOCMySkills()
{
	loadData();
}

AOCMySkills::~AOCMySkills()
{
}


//////////////////////////////////////////////////////////////////////
// Load data from disk
//////////////////////////////////////////////////////////////////////

void AOCMySkills::loadData()
{
	QDir AOCDir( QDir::currentDirPath() );
	QFile SkillsFile( AOCDir.filePath( "MySkills.cfg" ) );
	if( SkillsFile.open( IO_ReadOnly ) )
	{
		aocDebug( "AOCMySkills::loadData(): Loading skills file..." );
		QDataStream SkillsData( &SkillsFile );

		Q_UINT32 magic;
		SkillsData >> magic;
		if( magic != 0xA01EDA04)
		{
			qWarning( "BAD MAGIC NUMBER!" );
			SkillsFile.close();
			return;
		}

		Q_UINT16 skills;
		SkillsData >> skills;
		aocDebug( "AOCMySkills::loadData(): Specified skills: %d", skills );
		for( int x = 0; x < (int) skills; x++ )
		{
			AOCSkill newSkill;
			SkillsData >> newSkill.Skill;
			SkillsData >> newSkill.Value;
			aocDebug( "AOCMySkills::loadData(): * Skill #%d: %s (%d)", x, Names.getSkill( newSkill.Skill ).latin1(), newSkill.Value );
			Skills.append( newSkill );
		}

		aocDebug( "AOCMySkills::loadData(): Finished loading skills file" );
		SkillsFile.close();
	}
	else
	{
//		QMessageBox::critical( NULL, "AOCrafter", "Unable to load data file: File not found" );
		return;
	}
}


//////////////////////////////////////////////////////////////////////
// Save data to disk (only avaliable in admin)
//////////////////////////////////////////////////////////////////////

void AOCMySkills::saveData()
{
	QDir AOCDir( QDir::currentDirPath() );
	QFile SkillsFile( AOCDir.filePath( "MySkills.cfg" ) );
	if( SkillsFile.open( IO_WriteOnly ) )
	{
		aocDebug( "AOCMySkills::saveData(): Saving skills file..." );
		QDataStream SkillsData( &SkillsFile );
		SkillsData << 0xA01EDA04;

		aocDebug( "AOCMySkills::saveData(): Specified skills: %d", Skills.count() );
		SkillsData << (Q_UINT16) Skills.count();
		for( int x = 0; x < (int) Skills.count(); x++ )
		{
			AOCSkill newSkill = Skills[x];
			aocDebug( "AOCMySkills::saveData(): * Skill #%d: %s (%d)", x, Names.getSkill( newSkill.Skill ).latin1(), newSkill.Value );
			SkillsData << (Q_UINT32) newSkill.Skill;
			SkillsData << (Q_UINT16) newSkill.Value;
		}

		aocDebug( "AOCMySkills::saveData(): Finished saving skills file" );
		SkillsFile.close();
	}
}


//////////////////////////////////////////////////////////////////////
// Create empty data blocks (internal use only)
//////////////////////////////////////////////////////////////////////

AOCSkill AOCMySkills::blankSkill()
{
	AOCSkill ret;
	ret.Skill = 0;
	ret.Value = 0;
	return ret;
}


//////////////////////////////////////////////////////////////////////
// Fetch/find Skill
//////////////////////////////////////////////////////////////////////

int AOCMySkills::getSkills()
{
	return (int) Skills.count();
}

AOCSkill AOCMySkills::getSkill( int skill )
{
	if( skill < getSkills() && skill >= 0 )
	{
		return Skills[skill];
	}
	return blankSkill();
}

AOCSkill AOCMySkills::getSkillByID( Q_UINT32 skill )
{
	for( int s = 0; s < getSkills(); s++ )
	{
		if( Skills[s].Skill == skill )
		{
			return Skills[s];
		}
	}
	return blankSkill();
}

int AOCMySkills::getSkillIndexByID( Q_UINT32 skill )
{
	for( int s = 0; s < getSkills(); s++ )
	{
		if( Skills[s].Skill == skill )
		{
			return s;
		}
	}
	return -1;
}

void AOCMySkills::setSkill( Q_UINT32 ID, Q_UINT16 value )
{
	AOCSkill skill = getSkillByID( ID );
	int index = getSkillIndexByID( ID );
	if( index != -1 )
	{
		Skills.remove( Skills.at( index ) );
	}
	if( value > 0 )
	{
		skill.Skill = ID;
		skill.Value = value;
		Skills.append( skill );
	}
}
