// aoc_proc.cpp: implementation of the AOCProcData class.
//
//////////////////////////////////////////////////////////////////////

#include "aoc_proc.h"
#include "aoc_names.h"
#include <qmessagebox.h>
#include <qfile.h>
#include <qdir.h>


//////////////////////////////////////////////////////////////////////
// Static declarations
//////////////////////////////////////////////////////////////////////

int AOCProcData::CurrentGroup = -1;
int AOCProcData::CurrentProcess = -1;
int AOCProcData::CurrentStep = -1;


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

AOCProcData::AOCProcData()
{
	loadData();
}

AOCProcData::~AOCProcData()
{
}


//////////////////////////////////////////////////////////////////////
// Load data from disk
//////////////////////////////////////////////////////////////////////

void AOCProcData::loadData()
{
	QDir AOCDir( QDir::currentDirPath() );
	QFile ProcFile( AOCDir.filePath( "AOCrafter.dat" ) );
	if( ProcFile.open( IO_ReadOnly ) )
	{
		QDataStream ProcData( &ProcFile );

		Q_UINT32 magic;
		ProcData >> magic;
		if( magic != 0xA01EDA04)
		{
			qWarning( "BAD MAGIC NUMBER!" );
			ProcFile.close();
			return;
		}

		Q_UINT16 items;
		ProcData >> items;
		aocDebug( "AOCProcData::loadData(): Global Items: %d", items );
		for( int x = 0; x < (int) items; x++ )
		{
			AOCItem newItem;
			ProcData >> newItem.AOC_ID;
			ProcData >> newItem.Name;
			ProcData >> newItem.MinQL;
			ProcData >> newItem.MaxQL;
			ProcData >> newItem.AOdbID;
			aocDebug( "AOCProcData::loadData(): * Item #%d: %s (%d-%d)", x, newItem.Name.latin1(), newItem.MinQL, newItem.MaxQL );
			GlobalItems.append( newItem );
		}

		ProcData >> lastID;
		aocDebug( "AOCProcData::loadData(): Last ID: %d", lastID );

		Q_UINT16 grps;
		ProcData >> grps;
		aocDebug( "AOCProcData::loadData(): ProcGroups: %d", grps );

		for( int i = 0; i < (int) grps; i++ )
		{
			AOCProcGrp newGrp;
			ProcData >> newGrp.Name;
			aocDebug( "AOCProcData::loadData(): * Group #%d: %s", i, newGrp.Name.latin1() );

			Q_UINT16 procs;
			ProcData >> procs;
			aocDebug( "AOCProcData::loadData():   + Procs: %d", procs );

			for( int j = 0; j < (int) procs; j++ )
			{
				AOCProc newProc;
				ProcData >> newProc.Name;
				aocDebug( "AOCProcData::loadData():     * Proc #%d: %s", j, newProc.Name.latin1() );

				ProcData >> newProc.MaxQL;
				ProcData >> newProc.MinQL;
				aocDebug( "AOCProcData::loadData():       > Max QL: %d, Min QL: %d", newProc.MaxQL, newProc.MinQL );

				Q_UINT16 items;
				ProcData >> items;
				aocDebug( "AOCProcData::loadData():       + Items: %d", items );
				for( int k = 0; k < (int) items; k++ )
				{
					AOCItem newItem;
					ProcData >> newItem.AOC_ID;
					ProcData >> newItem.Name;
					ProcData >> newItem.MinQL;
					ProcData >> newItem.MaxQL;
					ProcData >> newItem.AOdbID;
					aocDebug( "AOCProcData::loadData():         * Item #%d: %s (%d-%d)", k, newItem.Name.latin1(), newItem.MinQL, newItem.MaxQL );
					newProc.Items.append( newItem );
				}

				Q_UINT16 steps;
				ProcData >> steps;
				aocDebug( "AOCProcData::loadData():       + Steps: %d", steps );

				for( int l = 0; l < (int) steps; l++ )
				{
					AOCStep newStep;
					aocDebug( "AOCProcData::loadData():         * Step #%d:", l );

					ProcData >> newStep.Source.AOC_ID;
					ProcData >> newStep.Source.Multiplier;

					ProcData >> newStep.Source.Flags;
					aocDebug( "AOCProcData::loadData():           > Source ID=%d  (x%.2f) (flags=%d)", newStep.Source.AOC_ID, newStep.Source.Multiplier, newStep.Source.Flags );

					ProcData >> newStep.Target.AOC_ID;
					ProcData >> newStep.Target.Multiplier;

					ProcData >> newStep.Target.Flags;
					aocDebug( "AOCProcData::loadData():           > Target ID=%d  (x%.2f) (flags=%d)", newStep.Target.AOC_ID, newStep.Target.Multiplier, newStep.Target.Flags );

					ProcData >> newStep.Result.AOC_ID;
					ProcData >> newStep.Result.Multiplier;

					aocDebug( "AOCProcData::loadData():           > Result ID=%d  (x%.2f)", newStep.Result.AOC_ID, newStep.Result.Multiplier );

					ProcData >> newStep.Count;
					aocDebug( "AOCProcData::loadData():           > Count=%d", newStep.Count );
					ProcData >> newStep.Note;
					aocDebug( "AOCProcData::loadData():           > Note: %s", newStep.Note.latin1() );

					Q_UINT16 skills;
					ProcData >> skills;
					aocDebug( "AOCProcData::loadData():           + Skills: %d", skills );

					for( int m = 0; m < (int) skills; m++ )
					{
						AOCReq newSkill;
						ProcData >> newSkill.AOC_ID;
						ProcData >> newSkill.Multiplier;
						aocDebug( "AOCProcData::loadData():             * Skill #%d: %d - x%.2f",
							m, newSkill.AOC_ID, newSkill.Multiplier );

						newStep.Skills.append( newSkill );
					}
					newProc.Steps.append( newStep );
				}
				newGrp.Procs.append( newProc );
			}
			ProcGroups.append( newGrp );
		}

		aocDebug( "AOCProcData::loadData(): Finished loading data file" );
		ProcFile.close();
	}
	else
	{
		return;
	}
}


//////////////////////////////////////////////////////////////////////
// Save data to disk (only avaliable in admin)
//////////////////////////////////////////////////////////////////////

void AOCProcData::saveData()
{
	QDir AOCDir( QDir::currentDirPath() );
	QFile ProcFile( AOCDir.filePath( "AOCrafter.dat" ) );
	AOCDir.remove( "AOCrafter.bak", FALSE );
	AOCDir.rename( "AOCrafter.dat", "AOCrafter.bak", FALSE );
	if( ProcFile.open( IO_WriteOnly ) )
	{
		aocDebug( "AOCProcData::saveData(): Saving data file..." );
		QDataStream ProcData( &ProcFile );
		ProcData << 0xA01EDA04;

		aocDebug( "AOCProcData::saveData(): Global Items: %d", GlobalItems.count() );
		ProcData << (Q_UINT16) GlobalItems.count();
		for( int x = 0; x < (int) GlobalItems.count(); x++ )
		{
			AOCItem newItem = GlobalItems[x];
			aocDebug( "AOCProcData::saveData(): * Item #%d: %s", x, newItem.Name.latin1() );
			ProcData << (Q_UINT32) newItem.AOC_ID;
			ProcData << newItem.Name;
			ProcData << (Q_UINT16) newItem.MinQL;
			ProcData << (Q_UINT16) newItem.MaxQL;
			ProcData << (Q_UINT32) newItem.AOdbID;
		}

		aocDebug( "AOCProcData::saveData(): Last ID: %d", lastID );
		ProcData << (Q_UINT32) lastID;

		aocDebug( "AOCProcData::saveData(): Groups: %d", ProcGroups.count() );
		ProcData << (Q_UINT16) ProcGroups.count();
		for( int i = 0; i < (int) ProcGroups.count(); i++ )
		{
			AOCProcGrp newGrp = ProcGroups[i];
			aocDebug( "AOCProcData::saveData(): * Group #%d: %s", i, newGrp.Name.latin1() );
			ProcData << newGrp.Name;

			aocDebug( "AOCProcData::saveData():   + Procs: %d", newGrp.Procs.count() );
			ProcData << (Q_UINT16) newGrp.Procs.count();
			for( int j = 0; j < (int) newGrp.Procs.count(); j++ )
			{
				AOCProc newProc = newGrp.Procs[j];
				aocDebug( "AOCProcData::saveData():     * Proc #%d: %s", j, newProc.Name.latin1() );
				ProcData << newProc.Name;
				aocDebug( "AOCProcData::saveData():       > Max QL: %d, Min QL: %d", newProc.MaxQL, newProc.MinQL );
				ProcData << (Q_UINT16) newProc.MaxQL;
				ProcData << (Q_UINT16) newProc.MinQL;

				aocDebug( "AOCProcData::saveData():       + Items: %d", newProc.Items.count() );
				ProcData << (Q_UINT16) newProc.Items.count();
				for( int k = 0; k < (int) newProc.Items.count(); k++ )
				{
					AOCItem newItem = newProc.Items[k];
					aocDebug( "AOCProcData::saveData():         * Item #%d: %s", k, newItem.Name.latin1() );
					ProcData << (Q_UINT32) newItem.AOC_ID;
					ProcData << newItem.Name;
					ProcData << (Q_UINT16) newItem.MinQL;
					ProcData << (Q_UINT16) newItem.MaxQL;
					ProcData << (Q_UINT32) newItem.AOdbID;
				}

				aocDebug( "AOCProcData::saveData():       + Steps: %d", newProc.Steps.count() );
				ProcData << (Q_UINT16) newProc.Steps.count();
				for( int l = 0; l < (int) newProc.Steps.count(); l++ )
				{
					AOCStep newStep = newProc.Steps[l];
					aocDebug( "AOCProcData::saveData():         * Step #%d:", l );

					aocDebug( "AOCProcData::saveData():           > Source ID=%d  (x%.2f) (flags=%d)", newStep.Source.AOC_ID, newStep.Source.Multiplier, newStep.Source.Flags );
					ProcData << (Q_UINT32) newStep.Source.AOC_ID;
					ProcData << (double) newStep.Source.Multiplier;
					ProcData << (Q_INT8) newStep.Source.Flags;

					aocDebug( "AOCProcData::saveData():           > Target ID=%d  (x%.2f) (flags=%d)", newStep.Target.AOC_ID, newStep.Target.Multiplier, newStep.Target.Flags );
					ProcData << (Q_UINT32) newStep.Target.AOC_ID;
					ProcData << (double) newStep.Target.Multiplier;
					ProcData << (Q_INT8) newStep.Target.Flags;

					aocDebug( "AOCProcData::saveData():           > Result ID=%d  (x%.2f)", newStep.Result.AOC_ID, newStep.Result.Multiplier );
					ProcData << (Q_UINT32) newStep.Result.AOC_ID;
					ProcData << (double) newStep.Result.Multiplier;

					aocDebug( "AOCProcData::saveData():           > Count=%d", newStep.Count );
					ProcData << (Q_UINT8) newStep.Count;
					aocDebug( "AOCProcData::saveData():           > Note: %s", newStep.Note.latin1() );
					ProcData << newStep.Note;

					aocDebug( "AOCProcData::saveData():           + Skills: %d", newStep.Skills.count() );
					ProcData << (Q_UINT16) newStep.Skills.count();
					for( int m = 0; m < (int) newStep.Skills.count(); m++ )
					{
						AOCReq newSkill = newStep.Skills[m];
						aocDebug( "AOCProcData::saveData():             * Skill #%d: %d - x%.2f",
							m, newSkill.AOC_ID, newSkill.Multiplier );
						ProcData << (Q_UINT32) newSkill.AOC_ID;
						ProcData << (double) newSkill.Multiplier;
					}
				}
			}
		}

		aocDebug( "AOCProcData::saveData(): Finished saving data file" );
		ProcFile.close();
	}
}


//////////////////////////////////////////////////////////////////////
// Re-read data from disc
//////////////////////////////////////////////////////////////////////

void AOCProcData::reloadData()
{
	ProcGroups.clear();
	loadData();

	CurrentGroup = -1;
	CurrentProcess = -1;
	CurrentStep = -1;
}


//////////////////////////////////////////////////////////////////////
// Sort data lists (only avaliable in admin)
//////////////////////////////////////////////////////////////////////

void AOCProcData::sortGroupList()
{
	ProcGrpList::Iterator it;
	ProcGrpList newList;

	aocDebug( "AOCProcData::sortGroupList(): Sorting ProcGroupList:" );
	for( it = ProcGroups.begin(); it != ProcGroups.end(); ++it )
	{
		if( newList.isEmpty() )
		{
			newList.append( (*it) );
			aocDebug( "AOCProcData::sortGroupList(): %s appended.", (*it).Name.latin1() );
		}
		else
		{
			int sorted = FALSE;
			ProcGrpList::Iterator zit;
			for( zit = newList.begin(); zit != newList.end() && sorted == FALSE; ++zit )
			{
				if( int r = (*zit).Name.compare( (*it).Name ) > 0 )
				{
					aocDebug( "AOCProcData::sortGroupList(): %s comes before %s (r = %d)", (*it).Name.latin1(), (*zit).Name.latin1(), r );
					newList.insert( zit, (*it) );
					sorted = TRUE;
				}
			}
			if( sorted == FALSE )
			{
				newList.append( (*it) );
				aocDebug ( "AOCProcData::sortGroupList(): %s appended.", (*it).Name.latin1() );
			}
		}
	}
	aocDebug( "AOCProcData::sortGroupList(): List sorted, saving..." );
	ProcGroups.clear();
	ProcGroups = newList;
	aocDebug( "AOCProcData::sortGroupList(): Sorting Complete!" );
}

void AOCProcData::sortProcessList()
{
	ProcList::Iterator it;
	ProcList newList;

	aocDebug( "AOCProcData::sortProcessList(): Sorting ProcessList (%s):", ProcGroups[CurrentGroup].Name.latin1() );
	for( it = ProcGroups[CurrentGroup].Procs.begin(); it != ProcGroups[CurrentGroup].Procs.end(); ++it )
	{
		if( newList.isEmpty() )
		{
			newList.append( (*it) );
			aocDebug( "AOCProcData::sortProcessList(): %s appended.", (*it).Name.latin1() );
		}
		else
		{
			int sorted = FALSE;
			ProcList::Iterator zit;
			for( zit = newList.begin(); zit != newList.end() && sorted == FALSE; ++zit )
			{
				if( int r = (*zit).Name.compare( (*it).Name ) > 0 )
				{
					aocDebug( "AOCProcData::sortProcessList(): %s comes before %s (r = %d)", (*it).Name.latin1(), (*zit).Name.latin1(), r );
					newList.insert( zit, (*it) );
					sorted = TRUE;
				}
			}
			if( sorted == FALSE )
			{
				newList.append( (*it) );
				aocDebug ( "AOCProcData::sortProcessList(): %s appended.", (*it).Name.latin1() );
			}
		}
	}
	aocDebug( "AOCProcData::sortProcessList(): List sorted, saving..." );
	ProcGroups[CurrentGroup].Procs.clear();
	ProcGroups[CurrentGroup].Procs = newList;
	aocDebug( "AOCProcData::sortProcessList(): Sorting Complete!" );
}

ReqsList AOCProcData::sortReqsList( ReqsList list, bool asc, bool isSkills )
{
	ReqsList::Iterator it;
	ReqsList newList;

	aocDebug( "AOCProcData::sortReqsList(): Sorting ReqsList (%p):", list );
	for( it = list.begin(); it != list.end(); ++it )
	{
		AOCItem item = isSkills ? blankItem() : Procs.getItemByID( (*it).AOC_ID );
		if( newList.isEmpty() )
		{
			newList.append( (*it) );
			aocDebug( "AOCProcData::sortReqsList(): %s appended.",
				isSkills ? Names.getSkill( (*it).AOC_ID ).latin1() : item.Name.latin1() );
		}
		else
		{
			int sorted = FALSE;
			ReqsList::Iterator zit;
			for( zit = newList.begin(); zit != newList.end() && sorted == FALSE; ++zit )
			{
				if( isSkills )
				{
					QString skill = Names.getSkill( (*zit).AOC_ID );
					if( int r = skill.compare( Names.getSkill( (*it).AOC_ID ) ) > 0 )
					{
						aocDebug( "AOCProcData::sortReqsList(): %s comes before %s (r = %d)", skill.latin1(), Names.getSkill( (*it).AOC_ID ).latin1(), r );
						newList.insert( zit, (*it) );
						sorted = TRUE;
					}
				}
				else
				{
					AOCItem item2 = Procs.getItemByID( (*zit).AOC_ID );
					if( int r = item2.Name.compare( item.Name ) > 0 )
					{
						aocDebug( "AOCProcData::sortReqsList(): %s comes before %s (r = %d)", item.Name.latin1(), item2.Name.latin1(), r );
						newList.insert( zit, (*it) );
						sorted = TRUE;
					}
				}
			}
			if( sorted == FALSE )
			{
				newList.append( (*it) );
				aocDebug ( "AOCProcData::sortReqsList(): %s appended.",
					isSkills ? Names.getSkill( (*it).AOC_ID ).latin1() : item.Name.latin1() );
			}
		}
	}
	aocDebug( "AOCProcData::sortReqsList(): Sorting Complete!" );
	list.clear();
	return newList;
}


//////////////////////////////////////////////////////////////////////
// Create empty data blocks (internal use only)
//////////////////////////////////////////////////////////////////////

AOCProc AOCProcData::blankProc()
{
	AOCProc ret;
	ret.Name = "";
	ret.MaxQL = 0;
	ret.MinQL = 0;
	return ret;
}

AOCStep AOCProcData::blankStep()
{
	AOCStep ret;
	ret.Source = blankReq();
	ret.Target = blankReq();
	ret.Result = blankReq();
	ret.Note = "";
	ret.Action = 0;
	return ret;
}

AOCItem AOCProcData::blankItem()
{
	AOCItem ret;
	ret.Name == "";
	ret.AOC_ID = 0;
	ret.AOdbID = 0;
	ret.MinQL = 0;
	ret.MaxQL = 0;
	return ret;
}

AOCReq AOCProcData::blankReq()
{
	AOCReq ret;
	ret.AOC_ID = 0;
	ret.Multiplier = 0.0;
	return ret;
}


//////////////////////////////////////////////////////////////////////
// Fetch/find Global Items
//////////////////////////////////////////////////////////////////////

int AOCProcData::getItems()
{
	return (int) GlobalItems.count();
}

ItemList AOCProcData::getItemList()
{
	return GlobalItems;
}

AOCItem AOCProcData::getItem( int item )
{
	if( item < getItems() && item >= 0 )
	{
		return GlobalItems[item];
	}
	return blankItem();
}

AOCItem AOCProcData::getItemByID( Q_UINT32 ID, int proc, int grp )
{
	for( int m = 0; m < getItems(); m++ )
	{
		if( GlobalItems[m].AOC_ID == ID )
		{
			return GlobalItems[m];
		}
	}
	for( int n = 0; n < getProcItems( proc, grp ); n++ )
	{
		if( ProcGroups[grp].Procs[proc].Items[n].AOC_ID == ID )
		{
			return ProcGroups[grp].Procs[proc].Items[n];
		}
	}
	return blankItem();
}

int AOCProcData::getItemIndexByID( Q_UINT32 ID, int proc, int grp )
{
	for( int m = 0; m < getItems(); m++ )
	{
		if( GlobalItems[m].AOC_ID == ID )
		{
			return m;
		}
	}
	for( int n = 0; n < getProcItems( proc, grp ); n++ )
	{
		if( ProcGroups[grp].Procs[proc].Items[n].AOC_ID == ID )
		{
			return n;
		}
	}
	return -1;
}

void AOCProcData::addItem( AOCItem item, int index )
{
	if( item.AOC_ID == 0 )
	{
		item.AOC_ID = nextFreeID();
	}

	switch( index )
	{
	case Append:
		GlobalItems.append( item );
		break;
	case Prepend:
		GlobalItems.prepend( item );
		break;
	case Replace:
		index = getItemIndexByID( item.AOC_ID );
		removeItem( index );
		/* This falls through to default to reinsert the item */
	default:
		GlobalItems.insert( GlobalItems.at( index ), item );
		break;
	}
}

void AOCProcData::removeItem( int item )
{
	if( item < getItems() && item >= 0 )
	{
		GlobalItems.remove( GlobalItems.at( item ) );
	}
}


//////////////////////////////////////////////////////////////////////
// Fetch/find Process Groups
//////////////////////////////////////////////////////////////////////

int AOCProcData::getProcGroups()
{
	return (int) ProcGroups.count();
}

QString AOCProcData::getProcGroup( int grp )
{
	if( grp < getProcGroups() && grp >= 0 )
	{
		CurrentGroup = grp;
		return ProcGroups[grp].Name;
	}
	return "Unknown";
}

void AOCProcData::setProcGroup( int grp )
{
	if( grp < getProcGroups() && grp >= 0 )
	{
		CurrentGroup = grp;
	}
}

void AOCProcData::addProcGroup( AOCProcGrp group, int index )
{
	switch( index )
	{
	case Append:
		ProcGroups.append( group );
		break;
	case Prepend:
		ProcGroups.prepend( group );
		break;
	case Replace:
		index = CurrentGroup;
		group.Procs = ProcGroups[index].Procs;
		removeProcGroup( index );
		/* This falls through to default to reinsert the group */
	default:
		ProcGroups.insert( ProcGroups.at( index ), group );
		break;
	}
}

void AOCProcData::removeProcGroup( int grp )
{
	if( grp < getProcGroups() && grp >= 0 )
	{
		ProcGroups.remove( ProcGroups.at( grp ) );
	}
}


//////////////////////////////////////////////////////////////////////
// Fetch/find Processes
//////////////////////////////////////////////////////////////////////

int AOCProcData::getProcesses( int grp )
{
	if( grp < getProcGroups() && grp >= 0 )
	{
		return (int) ProcGroups[grp].Procs.count();
	}
	return -1;
}

AOCProc AOCProcData::getProcess( int proc, int grp )
{
	if( proc < getProcesses( grp ) && proc >= 0 )
	{
		CurrentProcess = proc;
		return ProcGroups[grp].Procs[proc];
	}
	return blankProc();
}

void AOCProcData::setProcess( int proc )
{
	if( proc < getProcesses() && proc >= 0 )
	{
		CurrentProcess = proc;
	}
}

void AOCProcData::addProcess( AOCProc proc, int index, int grp )
{
	switch( index )
	{
	case Append:
		ProcGroups[grp].Procs.append( proc );
		break;
	case Prepend:
		ProcGroups[grp].Procs.prepend( proc );
		break;
	case Replace:
		index = CurrentProcess;
		proc.Items = ProcGroups[grp].Procs[index].Items;
		proc.Steps = ProcGroups[grp].Procs[index].Steps;
		removeProcess( index, grp );
		/* This falls through to default to reinsert the group */
	default:
		ProcGroups[grp].Procs.insert( ProcGroups[grp].Procs.at( index ), proc );
		break;
	}
}

void AOCProcData::removeProcess( int proc, int grp )
{
	if( proc < getProcesses( grp ) && proc >= 0 )
	{
		ProcGroups[grp].Procs.remove( ProcGroups[grp].Procs.at( proc ) );
	}
}


//////////////////////////////////////////////////////////////////////
// Fetch/find Process Items
//////////////////////////////////////////////////////////////////////

int AOCProcData::getProcItems( int proc, int grp )
{
	if( proc < getProcesses( grp ) && proc >= 0 )
	{
		return (int) ProcGroups[grp].Procs[proc].Items.count();
	}
	return -1;
}

AOCItem AOCProcData::getProcItem( int item, int proc, int grp )
{
	if( item < getProcItems( proc, grp ) && item >= 0 )
	{
		return ProcGroups[grp].Procs[proc].Items[item];
	}
	return blankItem();
}

void AOCProcData::addProcItem( AOCItem item, int index, int proc, int grp )
{
	if( item.AOC_ID == 0 )
	{
		item.AOC_ID = nextFreeID();
	}

	switch( index )
	{
	case Append:
		ProcGroups[grp].Procs[proc].Items.append( item );
		break;
	case Prepend:
		ProcGroups[grp].Procs[proc].Items.prepend( item );
		break;
	case Replace:
		index = getItemIndexByID( item.AOC_ID, proc, grp );
		removeProcItem( index, proc, grp );
		/* This falls through to default to reinsert the group */
	default:
		ProcGroups[grp].Procs[proc].Items.insert( ProcGroups[grp].Procs[proc].Items.at( index ), item );
		break;
	}
}

void AOCProcData::removeProcItem( int item, int proc, int grp )
{
	if( item < getProcItems( proc, grp ) && item >= 0 )
	{
		ProcGroups[grp].Procs[proc].Items.remove( ProcGroups[grp].Procs[proc].Items.at( item ) );
	}
}


//////////////////////////////////////////////////////////////////////
// Fetch/find Process Steps
//////////////////////////////////////////////////////////////////////

int AOCProcData::getSteps( int proc, int grp )
{
	if( proc < getProcesses( grp ) && proc >= 0 )
	{
		return (int) ProcGroups[grp].Procs[proc].Steps.count();
	}
	return -1;
}

AOCStep AOCProcData::getStep( int step, int proc, int grp )
{
	if( step < getSteps( proc, grp ) && step >= 0 )
	{
		CurrentStep = step;
		return ProcGroups[grp].Procs[proc].Steps[step];
	}
	return blankStep();
}

void AOCProcData::setStep( int step )
{
	if( step < getSteps() && step >= 0 )
	{
		CurrentStep = step;
	}
}

void AOCProcData::addStep( AOCStep step, int index, int proc, int grp )
{
	switch( index )
	{
	case Append:
		ProcGroups[grp].Procs[proc].Steps.append( step );
		break;
	case Prepend:
		ProcGroups[grp].Procs[proc].Steps.prepend( step );
		break;
	case Replace:
		index = CurrentStep;
		removeStep( index, proc, grp );
		/* This falls through to default to reinsert the group */
	default:
		ProcGroups[grp].Procs[proc].Steps.insert( ProcGroups[grp].Procs[proc].Steps.at( index ), step );
		break;
	}
}

void AOCProcData::removeStep( int step, int proc, int grp )
{
	if( step < getSteps( proc, grp ) && step >= 0 )
	{
		ProcGroups[grp].Procs[proc].Steps.remove( ProcGroups[grp].Procs[proc].Steps.at( step ) );
	}
}


//////////////////////////////////////////////////////////////////////
// Grab the next free Item ID (only avaliable in admin)
//////////////////////////////////////////////////////////////////////

Q_UINT32 AOCProcData::nextFreeID()
{
	if( lastID <= 0  || lastID >= 999999999 ) /* Big upper limit, I know, but gotta have it... */
	{
		lastID = 10;
	}
	return ++lastID;
}


//////////////////////////////////////////////////////////////////////
// Make a ReqsList structure unique - remove all duplicate entries
//////////////////////////////////////////////////////////////////////

ReqsList AOCProcData::makeUnique( ReqsList list )
{
	ReqsList::Iterator it;
	ReqsList newList;

	for( it = list.begin(); it != list.end(); ++it )
	{
		if( newList.isEmpty() )
		{
			aocDebug( "AOCProcData::makeUnique(): %i appended", (*it).AOC_ID );
			newList.append( (*it) );
		}
		else
		{
			int added = FALSE;
			ReqsList::Iterator nit;
			for( nit = newList.begin(); nit != newList.end() && added == FALSE; ++nit )
			{
				if( (*it).AOC_ID == (*nit).AOC_ID )
				{
					if( (*it).Multiplier >= (*nit).Multiplier )
					{
						(*it).Count = (*nit).Count + 1;
						aocDebug( "AOCProcData::makeUnique(): %d (%.2f) replaced with %d (%.2f)",
							(*nit).AOC_ID, (*nit).Multiplier, (*it).AOC_ID, (*it).Multiplier );
						newList.remove( nit );
						newList.append( (*it) );
					}
					added = TRUE;
				}
			}
			if( added == FALSE )
			{
				aocDebug( "AOCProcData::makeUnique(): %i appended", (*it).AOC_ID );
				newList.append( (*it) );
			}
		}
	}
	return newList;
}